const http = require('http');
const ERRORCODE=require("../const/error")
const server = http.createServer((req, res) => {
    var buffer;
    var log=require("../util/log")
    req.on("data", function (chunk) {
        if(!buffer){
            buffer=Buffer.from(chunk)
        }else{
            buffer=Buffer.concat([buffer,chunk])
        }
    });
    req.on("end",async function(){
        var methodName = req.url;
        var method = allMethods[methodName]
        if (!method) {
            log.warn("no "+methodName+" found")
            res.end(JSON.stringify({ c: ERRORCODE.NOMETHOD }));
            return;
        }
        try {
            var {func,max}=method;
            var data = buffer.toString()
            log.info(`call ${methodName} with ${data}`)
            try {
                var params = JSON.parse(data);
            } catch (e) {
                res.end(JSON.stringify({ c: ERRORCODE.BADDATA }));
                return;
            }
            var begin = Date.now();
            var result = await func(...params);

            var delta = Date.now() - begin;
            if (delta > max) {
                method.max = delta;
            }
            method.totalTime = delta;
            method.count++;
            var resStr
            if (result instanceof Buffer) {
                res.setHeader("content-type","Buffer")
                resStr= Buffer.concat([Uint8Array.from([ERRORCODE.SUCCESS]),result])
            } else {
                resStr=Buffer.from(JSON.stringify({ c: ERRORCODE.SUCCESS, o: result }));
            }
            log.info(`res ${methodName} ${resStr}`)
            res.end(resStr);
        }catch(a) {
            method.failed++;
            log.error("interal error of rpc",a)
            res.end(JSON.stringify({c:ERRORCODE.INSERNALERROR,e:a.message}))
        }
    })
});
var allMethods={}

// now that proxy is running
var started=false;
function register(name, func,desc){
    var log=require("../util/log")
    if(started){
        //todo;
        return;
    }
    var method=allMethods[name];
    if(!method){
        console.info("add "+name)
        log.info("add "+name)
        allMethods[name]={name,desc,func,count:0,max:0,totalTime:0,failed:0};
    }else{
        log.warn("failed to add "+name)
        //TODO报错
    }
}
function registers(funcs) {
    if (!Array.isArray(funcs)) {
        funcs = [funcs];
    }
    funcs.forEach(funcGroup => {
        for (var key in funcGroup) {
            var func = funcGroup[key];
            register(key, func[0], func[1]);
        }
    });
}
async function __internalList__() {
    return allMethods;
}
function start(options) {
    if(started){
        return;
    }
    register("/__internalList__",__internalList__);
    started=true;
    var host=options.host||"127.0.0.1"
    server.listen(options.port,host);
    server.keepAliveTimeout=0;
    if(options.ips){
        server.on("connection",(socket)=>{
            var remoteAddress=socket.remoteAddress;
            if(options.ips.indexOf(remoteAddress)<0){
                log.warn(`close connection from ${remoteAddress}`)
                socket.destroy();
            }
        });
    }
    return function(){
        server.close();
    }
}
module.exports={start,register,registers}
